@@ -7,7 +7,7 @@ use std::sync::atomic::{AtomicBool, Ordering};
 use ipc_channel::ipc::{self, IpcReceiver, IpcSender, OpaqueIpcReceiver, OpaqueIpcSender};
 use serde::{Deserialize, Serialize};
 
-use crate::error::Panic;
+use crate::error::PanicInfo;
 use crate::panic::{init_panic_hook, reset_panic_info, take_panic, BacktraceCapture};
 
 pub const ENV_NAME: &str = "__PROCSPAWN_CONTENT_PROCESS_ID";
@@ -36,6 +36,10 @@ impl Default for ProcConfig {
     }
 }
 
+pub fn mark_initialized() {
+    INITIALIZED.store(true, Ordering::SeqCst);
+}
+
 impl ProcConfig {
     /// Creates a default proc config.
     pub fn new() -> ProcConfig {
@@ -43,7 +47,7 @@ impl ProcConfig {
     }
 
     /// Attaches a callback that is used to initializes all processes.
-    pub fn config_callback<F: FnOnce() + 'static>(mut self, f: F) -> ProcConfig {
+    pub fn config_callback<F: FnOnce() + 'static>(&mut self, f: F) -> &mut Self {
         self.callback = Some(Box::new(f));
         self
     }
@@ -52,7 +56,7 @@ impl ProcConfig {
     ///
     /// The default behavior is that panics are caught and that a panic handler
     /// is installed.
-    pub fn panic_handling(mut self, enabled: bool) -> ProcConfig {
+    pub fn panic_handling(&mut self, enabled: bool) -> &mut Self {
         self.panic_handling = enabled;
         self
     }
@@ -64,21 +68,21 @@ impl ProcConfig {
     ///
     /// This requires the `backtrace` feature.
     #[cfg(feature = "backtrace")]
-    pub fn capture_backtraces(mut self, enabled: bool) -> ProcConfig {
+    pub fn capture_backtraces(&mut self, enabled: bool) -> &mut Self {
         self.capture_backtraces = enabled;
         self
     }
 
     /// Controls whether backtraces should be resolved.
     #[cfg(feature = "backtrace")]
-    pub fn resolve_backtraces(mut self, enabled: bool) -> ProcConfig {
+    pub fn resolve_backtraces(&mut self, enabled: bool) -> &mut Self {
         self.resolve_backtraces = enabled;
         self
     }
 
     /// Consumes the config and initializes the process.
-    pub fn init(mut self) {
-        INITIALIZED.store(true, Ordering::SeqCst);
+    pub fn init(&mut self) {
+        mark_initialized();
 
         if let Ok(token) = env::var(ENV_NAME) {
             // permit nested invocations
@@ -150,7 +154,7 @@ impl MarshalledCall {
     /// Marshalls the call.
     pub fn marshal<F, A, R>(
         args_receiver: IpcReceiver<A>,
-        return_sender: IpcSender<Result<R, Panic>>,
+        return_sender: IpcSender<Result<R, PanicInfo>>,
     ) -> MarshalledCall
     where
         F: FnOnce(A) -> R,
