@@ -1,37 +1,72 @@
-use std::collections::HashMap;
-use std::ffi::{OsStr, OsString};
+use std::ffi::OsStr;
+use std::fmt;
+use std::io;
+use std::process;
 use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
 use std::sync::{mpsc, Arc, Condvar, Mutex};
 use std::thread;
+use std::time::Duration;
 
 use ipc_channel::ipc;
 use serde::{Deserialize, Serialize};
 
 use crate::core::MarshalledCall;
 use crate::error::SpawnError;
-use crate::proc::{Builder, JoinHandle, JoinHandleInner};
+use crate::proc::{Builder, JoinHandle, JoinHandleInner, ProcCommon, ProcessHandleState};
 
 type WaitFunc = Box<dyn FnOnce() -> bool + Send>;
 type NotifyErrorFunc = Box<dyn FnMut(SpawnError) + Send>;
 
-pub struct ScheduledTask {
-    cancelled: AtomicBool,
-    process: AtomicUsize,
+#[derive(Debug)]
+pub struct PooledHandleState {
+    pub cancelled: AtomicBool,
+    pub process_handle_state: Mutex<Option<Arc<ProcessHandleState>>>,
 }
 
-impl ScheduledTask {
-    pub fn pid(&self) -> Option<u32> {
-        match self.process.load(Ordering::SeqCst) {
-            0 => None,
-            x => Some(x as u32),
+impl PooledHandleState {
+    pub fn kill(&self) {
+        self.cancelled.store(true, Ordering::SeqCst);
+        if let Some(ref process_handle_state) = *self.process_handle_state.lock().unwrap() {
+            process_handle_state.kill();
+        }
+    }
+}
+
+pub struct PooledHandle<T> {
+    waiter_rx: mpsc::Receiver<Result<T, SpawnError>>,
+    shared: Arc<PooledHandleState>,
+}
+
+impl<T> PooledHandle<T> {
+    pub fn process_handle_state(&self) -> Option<Arc<ProcessHandleState>> {
+        self.shared.process_handle_state.lock().unwrap().clone()
+    }
+
+    pub fn kill(&mut self) -> Result<(), SpawnError> {
+        self.shared.kill();
+        Ok(())
+    }
+}
+
+impl<T: Serialize + for<'de> Deserialize<'de>> PooledHandle<T> {
+    pub fn join(&mut self) -> Result<T, SpawnError> {
+        match self.waiter_rx.recv() {
+            Ok(Ok(rv)) => Ok(rv),
+            Ok(Err(err)) => Err(err),
+            Err(..) => Err(io::Error::new(io::ErrorKind::BrokenPipe, "process went away").into()),
         }
     }
 
-    pub fn kill(&self) {
-        self.cancelled.store(true, Ordering::SeqCst);
-        if let Some(pid) = self.pid() {
-            unsafe {
-                libc::kill(pid as i32, libc::SIGKILL);
+    pub fn join_timeout(&mut self, timeout: Duration) -> Result<T, SpawnError> {
+        match self.waiter_rx.recv_timeout(timeout) {
+            Ok(Ok(rv)) => Ok(rv),
+            Ok(Err(err)) => Err(err),
+            Err(mpsc::RecvTimeoutError::Timeout) => {
+                self.kill().ok();
+                Err(SpawnError::new_timeout())
+            }
+            Err(mpsc::RecvTimeoutError::Disconnected) => {
+                Err(io::Error::new(io::ErrorKind::BrokenPipe, "process went away").into())
             }
         }
     }
@@ -47,27 +82,54 @@ impl ScheduledTask {
 /// through the pool.
 ///
 /// When the process pool is dropped all processes are killed.
+///
+/// This requires the `pool` feature.
 pub struct Pool {
     sender: mpsc::Sender<(
         MarshalledCall,
-        Arc<ScheduledTask>,
+        Arc<PooledHandleState>,
         WaitFunc,
         NotifyErrorFunc,
     )>,
     shared: Arc<PoolShared>,
 }
 
+impl fmt::Debug for Pool {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_struct("Pool")
+            .field("size", &self.size())
+            .field("active_count", &self.active_count())
+            .field("queued_count", &self.queued_count())
+            .finish()
+    }
+}
+
 impl Pool {
     /// Creates the default pool.
     pub fn new(size: usize) -> Result<Pool, SpawnError> {
         Pool::builder(size).build()
     }
 
-    /// Creats a builder to customize pool creation.
+    /// Creates a builder to customize pool creation.
     pub fn builder(size: usize) -> PoolBuilder {
         PoolBuilder::new(size)
     }
 
+    /// Returns the size of the pool.
+    pub fn size(&self) -> usize {
+        self.shared.monitors.lock().unwrap().len()
+    }
+
+    /// Returns the number of jobs waiting to executed in the pool.
+    pub fn queued_count(&self) -> usize {
+        self.shared.queued_count.load(Ordering::Relaxed)
+    }
+
+    /// Returns the number of currently active threads.
+    pub fn active_count(&self) -> usize {
+        self.shared.active_count.load(Ordering::SeqCst)
+    }
+
     /// Spawns a closure into a process of the pool.
     pub fn spawn<
         F: FnOnce(A) -> R + Copy,
@@ -88,15 +150,15 @@ impl Pool {
         let error_waiter_tx = waiter_tx.clone();
         self.shared.queued_count.fetch_add(1, Ordering::SeqCst);
 
-        let task = Arc::new(ScheduledTask {
+        let shared = Arc::new(PooledHandleState {
             cancelled: AtomicBool::new(false),
-            process: AtomicUsize::new(0),
+            process_handle_state: Mutex::new(None),
         });
 
         self.sender
             .send((
                 call,
-                task.clone(),
+                shared.clone(),
                 Box::new(move || {
                     if let Ok(rv) = return_rx.recv() {
                         waiter_tx.send(rv.map_err(Into::into)).is_ok()
@@ -111,7 +173,7 @@ impl Pool {
             .ok();
 
         JoinHandle {
-            inner: Ok(JoinHandleInner::Pooled { waiter_rx, task }),
+            inner: Ok(JoinHandleInner::Pooled(PooledHandle { waiter_rx, shared })),
         }
     }
 
@@ -142,7 +204,7 @@ impl Pool {
     }
 
     /// Joins and shuts down.
-    pub fn shutdown(self) {
+    pub fn shutdown(&self) {
         if !self.shared.dead.load(Ordering::SeqCst) {
             self.join();
             self.kill();
@@ -158,7 +220,7 @@ impl Pool {
         }
         self.shared.dead.store(true, Ordering::SeqCst);
         for monitor in self.shared.monitors.lock().unwrap().iter_mut() {
-            if let Some(join_handle) = monitor.join_handle.lock().unwrap().take() {
+            if let Some(mut join_handle) = monitor.join_handle.lock().unwrap().take() {
                 join_handle.kill().ok();
             }
         }
@@ -172,9 +234,15 @@ impl Pool {
 }
 
 /// Utility to configure a pool.
+///
+/// This requires the `pool` feature.
+#[derive(Debug)]
 pub struct PoolBuilder {
     size: usize,
-    vars: HashMap<OsString, OsString>,
+    disable_stdin: bool,
+    disable_stdout: bool,
+    disable_stderr: bool,
+    common: ProcCommon,
 }
 
 impl PoolBuilder {
@@ -186,57 +254,35 @@ impl PoolBuilder {
     fn new(size: usize) -> PoolBuilder {
         PoolBuilder {
             size,
-            vars: std::env::vars_os().collect(),
+            disable_stdin: false,
+            disable_stdout: false,
+            disable_stderr: false,
+            common: ProcCommon::default(),
         }
     }
 
-    /// Set an environment variable in the spawned process.
-    ///
-    /// Equivalent to `Command::env`
-    pub fn env<K, V>(&mut self, key: K, val: V) -> &mut Self
-    where
-        K: AsRef<OsStr>,
-        V: AsRef<OsStr>,
-    {
-        self.vars
-            .insert(key.as_ref().to_owned(), val.as_ref().to_owned());
+    define_common_methods!();
+
+    /// Redirects stdin to `/dev/null`.
+    pub fn disable_stdin(&mut self) -> &mut Self {
+        self.disable_stdin = true;
         self
     }
 
-    /// Set environment variables in the spawned process.
-    ///
-    /// Equivalent to `Command::envs`
-    pub fn envs<I, K, V>(&mut self, vars: I) -> &mut Self
-    where
-        I: IntoIterator<Item = (K, V)>,
-        K: AsRef<OsStr>,
-        V: AsRef<OsStr>,
-    {
-        self.vars.extend(
-            vars.into_iter()
-                .map(|(k, v)| (k.as_ref().to_owned(), v.as_ref().to_owned())),
-        );
+    /// Redirects stdout to `/dev/null`.
+    pub fn disable_stdout(&mut self) -> &mut Self {
+        self.disable_stdout = true;
         self
     }
 
-    /// Removes an environment variable in the spawned process.
-    ///
-    /// Equivalent to `Command::env_remove`
-    pub fn env_remove<K: AsRef<OsStr>>(&mut self, key: K) -> &mut Self {
-        self.vars.remove(key.as_ref());
-        self
-    }
-
-    /// Clears all environment variables in the spawned process.
-    ///
-    /// Equivalent to `Command::env_clear`
-    pub fn env_clear(&mut self) -> &mut Self {
-        self.vars.clear();
+    /// Redirects stderr to `/dev/null`.
+    pub fn disable_stderr(&mut self) -> &mut Self {
+        self.disable_stderr = true;
         self
     }
 
     /// Creates the pool.
-    pub fn build(self) -> Result<Pool, SpawnError> {
+    pub fn build(&mut self) -> Result<Pool, SpawnError> {
         let (tx, rx) = mpsc::channel();
 
         let shared = Arc::new(PoolShared {
@@ -272,7 +318,7 @@ struct PoolShared {
     call_receiver: Mutex<
         mpsc::Receiver<(
             MarshalledCall,
-            Arc<ScheduledTask>,
+            Arc<PooledHandleState>,
             WaitFunc,
             NotifyErrorFunc,
         )>,
@@ -315,19 +361,32 @@ fn spawn_worker(
     let current_call_tx = Arc::new(Mutex::new(None::<ipc::IpcSender<MarshalledCall>>));
 
     let spawn = Arc::new(Mutex::new({
-        let vars = builder.vars.clone();
+        let disable_stdin = builder.disable_stdin;
+        let disable_stdout = builder.disable_stdout;
+        let disable_stderr = builder.disable_stderr;
+        let common = builder.common.clone();
         let join_handle = join_handle.clone();
         let current_call_tx = current_call_tx.clone();
         move || {
             let (call_tx, call_rx) = ipc::channel::<MarshalledCall>().unwrap();
-            *join_handle.lock().unwrap() =
-                Some(Builder::new().envs(vars.clone()).spawn(call_rx, |rx| {
-                    while let Ok(call) = rx.recv() {
-                        // we never want panic handling here as we're going to
-                        // defer this to the process'.
-                        call.call(false);
-                    }
-                }));
+            let mut builder = Builder::new();
+            builder.common(common.clone());
+            if disable_stdin {
+                builder.stdin(process::Stdio::null());
+            }
+            if disable_stdout {
+                builder.stdout(process::Stdio::null());
+            }
+            if disable_stderr {
+                builder.stderr(process::Stdio::null());
+            }
+            *join_handle.lock().unwrap() = Some(builder.spawn(call_rx, |rx| {
+                while let Ok(call) = rx.recv() {
+                    // we never want panic handling here as we're going to
+                    // defer this to the process'.
+                    call.call(false);
+                }
+            }));
             *current_call_tx.lock().unwrap() = Some(call_tx);
         }
     }));
@@ -365,7 +424,7 @@ fn spawn_worker(
                     break;
                 }
 
-                let (call, scheduled_task, wait_func, mut err_func) = {
+                let (call, state, wait_func, mut err_func) = {
                     // Only lock jobs for the time it takes
                     // to get a job, not run it.
                     let lock = shared
@@ -382,13 +441,11 @@ fn spawn_worker(
                 shared.queued_count.fetch_sub(1, Ordering::SeqCst);
 
                 // this task was already cancelled, no need to execute it
-                if scheduled_task.cancelled.load(Ordering::SeqCst) {
+                if state.cancelled.load(Ordering::SeqCst) {
                     err_func(SpawnError::new_cancelled());
                 } else {
                     if let Some(ref mut handle) = *join_handle.lock().unwrap() {
-                        scheduled_task
-                            .process
-                            .store(handle.pid().unwrap_or(0) as usize, Ordering::SeqCst);
+                        *state.process_handle_state.lock().unwrap() = handle.process_handle_state();
                     }
 
                     let mut restart = false;
@@ -410,6 +467,8 @@ fn spawn_worker(
                         restart = true;
                     }
 
+                    *state.process_handle_state.lock().unwrap() = None;
+
                     if restart {
                         check_for_restart(&mut err_func);
                     }
