@@ -2,6 +2,8 @@
 use std::process::Command;
 use std::sync::atomic::{AtomicBool, Ordering};
 
+use crate::core::mark_initialized;
+
 static TEST_MODE: AtomicBool = AtomicBool::new(false);
 
 /// Supports the use of procspawn in tests.
@@ -19,12 +21,13 @@ static TEST_MODE: AtomicBool = AtomicBool::new(false);
 macro_rules! enable_test_support {
     () => {
         #[ctor::ctor]
+        #[used]
         fn __procspawn_test_support_init() {
             $crate::testsupport::enable();
         }
 
         #[test]
-        fn __procspawn_test_helper() {
+        fn procspawn_test_helper() {
             $crate::init();
         }
     };
@@ -32,11 +35,12 @@ macro_rules! enable_test_support {
 
 pub fn enable() {
     TEST_MODE.store(true, Ordering::SeqCst);
+    mark_initialized();
 }
 
 pub fn update_command_for_tests(cmd: &mut Command) {
     if TEST_MODE.load(Ordering::SeqCst) {
-        cmd.arg("__procspawn_test_helper");
+        cmd.arg("procspawn_test_helper");
         cmd.arg("--exact");
         cmd.arg("--test-threads=1");
         cmd.arg("-q");
